#include <SPI.h>

#include <Servo.h>
#include <DHT.h>
#include <LiquidCrystal.h>

/* push buttons */
const byte openStepWindowPin = A0;
const byte openFullWindowPin = A1;
const byte closeStepWindowPin = A2;
const byte closeFullWindowPin = A3;
const byte controlPin = A4;

/* temperature sensor */
const byte dhtpin = 11;
const byte dhttype = DHT22;
DHT dht(dhtpin, dhttype);

/* window */
int angleWindow = 0, angleStep = 1.5;
int minAngleWindow = 0, maxAngleWindow = 60;

/* servo motor */
Servo window;
int windowPin = 10;

/* LCD */
LiquidCrystal lcd(8, 9, 4, 5, 6, 7);

void setup() {
  Serial.begin(9600);

  /* init push buttons */
  pinMode(openStepWindowPin, OUTPUT);
  digitalWrite(openStepWindowPin, HIGH);

  pinMode(openFullWindowPin, OUTPUT);
  digitalWrite(openFullWindowPin, HIGH);
  
  pinMode(closeStepWindowPin, OUTPUT);
  digitalWrite(closeStepWindowPin, HIGH);
  
  pinMode(closeFullWindowPin, OUTPUT);
  digitalWrite(closeFullWindowPin, HIGH);

  pinMode(controlPin, OUTPUT);
  digitalWrite(controlPin, HIGH);

  /* init temperature sensor */
  dht.begin();
  float h = dht.readHumidity();
  float t = dht.readTemperature();
  if (isnan(h) || isnan(t)) {
      Serial.println("Failed to read from sensor\n");
      return;
  }

  /* init servo motor */
  window.attach(windowPin);
  angleWindow = t * 0.75 + h * 0.35;
  if (angleWindow > maxAngleWindow)
    angleWindow = maxAngleWindow;
  window.write(angleWindow);

  /* init LCD */
  lcd.begin(16, 2);
  lcd.print("Temp: ");
  lcd.print(t);
  lcd.setCursor(0, 1);
  lcd.print("Hum: ");
  lcd.print(h);
  
}

void loop() {
  if (digitalRead(controlPin) == LOW) {
    float h = dht.readHumidity();
    float t = dht.readTemperature();
    angleWindow = t * 0.75 + h * 0.35;
    delay(10);

    Serial.println("Resetting window");
  } else if (digitalRead(openStepWindowPin) == LOW && angleWindow + angleStep <= maxAngleWindow) {
    angleWindow += angleStep;
    delay(10);

    Serial.println("Opening window");
  } else if (digitalRead(openFullWindowPin) == LOW) {
    angleWindow = maxAngleWindow;
    delay(10);

    Serial.println("Fully opened window");
  } else if (digitalRead(closeStepWindowPin) == LOW && angleWindow - angleStep >= 0) {
    angleWindow -= angleStep;
    delay(10);

    Serial.println("Closing window");
  } else if (digitalRead(closeFullWindowPin) == LOW) {
    angleWindow = 0;
    delay(10);

    Serial.println("Closed window");
  }
    
  window.write(angleWindow);
}

